import cv2
import numpy as np
import glob

# Load mapping files
mapping_files = sorted(glob.glob("mass_*.ppm"))
current_index = 0


def load_mapping(filename):
    with open(filename, "r") as f:
        lines = f.readlines()

    header = list(map(float, lines[0].split()))
    obj_px_w, obj_px_h, scr_px_w, scr_px_h = map(int, header[:4])

    mapping_x = np.zeros((scr_px_h, scr_px_w), dtype=np.float32)
    mapping_y = np.zeros((scr_px_h, scr_px_w), dtype=np.float32)

    for j in range(scr_px_h):
        for i in range(scr_px_w):
            px, py = map(float, lines[1 + j * scr_px_w + i].split())
            mapping_x[j, i] = px
            mapping_y[j, i] = py

    return obj_px_w, obj_px_h, scr_px_w, scr_px_h, mapping_x, mapping_y


# Start with the first mapping file
obj_px_w, obj_px_h, scr_px_w, scr_px_h, mapping_x, mapping_y = load_mapping(mapping_files[current_index])

# Open camera
cap = cv2.VideoCapture(0)
if not cap.isOpened():
    print("Error: The camera could not be opened.")
    exit()

while True:
    ret, frame = cap.read()
    if not ret:
        print("Error: Did not receive frame.")
        break

    frame = cv2.resize(frame, (obj_px_w, obj_px_h))
    distorted = cv2.remap(frame, mapping_y, mapping_x, interpolation=cv2.INTER_LINEAR, borderMode=cv2.BORDER_CONSTANT)
    mirrored = cv2.flip(distorted, 1)

    height = 1000 * 10 / 16
    remove_px = int((1000 - height)/2)
    cropped = mirrored[remove_px:1000-remove_px, :]

    cv2.imshow("Black Hole Camera", cropped)

    key = cv2.waitKey(1)
    #print(f"Key pressed: {key}")
    if key == ord('q'):
        break
    elif key == 2:
        current_index = (current_index - 1) % len(mapping_files)
        obj_px_w, obj_px_h, scr_px_w, scr_px_h, mapping_x, mapping_y = load_mapping(mapping_files[current_index])
    elif key == 3:
        current_index = (current_index + 1) % len(mapping_files)
        obj_px_w, obj_px_h, scr_px_w, scr_px_h, mapping_x, mapping_y = load_mapping(mapping_files[current_index])

cap.release()
cv2.destroyAllWindows()